/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include <glib.h>

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>

#include "ishzilla-ipc.h"
#include "ishzilla-app.h"

extern IshzillaApp *app;

void ishzilla_check_running(gint *argc,gchar ***argv)
{
	FILE *fd;
	gchar *file;
	gchar line[32];
	gint pid;

	file = g_strconcat(g_get_home_dir(),"/.ishzilla/.ishzilla-pid",NULL);

	fd = fopen(file,"r");
	g_free(file);

	if (fd == NULL) {

		return; /* start a new one */
	}

	if (fgets(line,32,fd) == NULL) {
g_warning("pidfile exists but seems to be empty");
		return;
	}

	fclose(fd);
	
	pid = atoi(line);

	if (pid > 0 && kill(pid,0) == 0) { /* its alive */
		if (*argc < 2)  {
			kill(pid,SIGUSR1);
		} else { /* write url to file, read when launched */
			
			file = g_strconcat(g_get_home_dir(),"/.ishzilla/.ishzilla-uri",NULL);
			fd = fopen(file,"w");
			g_free(file);

			if (fd == NULL) {
g_message("Could not open url file");
				kill(pid,SIGUSR1);
			} else {
				fprintf(fd,"%s",(*argv)[1]);

				fclose(fd);
	
				kill(pid,SIGUSR1);
			}
		}

		ishzilla_preferences_finish();

		exit(0);
	}
/* else, it crashed & didnt remove the pid file so start anyway */

	return;
}

void ishzilla_set_running(void)
{
	FILE *fd;
	gchar *file;
	pid_t pid;

	pid = getpid();

	file = g_strconcat(g_get_home_dir(),"/.ishzilla/.ishzilla-pid",NULL);

	fd = fopen(file,"w");
	g_free(file);

	if (fd == NULL) {
g_warning("Could not write pid");
		return;
	}

	fprintf(fd,"%d",pid);
	fclose(fd);

	return;
}

void ishzilla_end_running(void)
{
	gchar *file = g_strconcat(g_get_home_dir(),"/.ishzilla/.ishzilla-pid",NULL);

	unlink(file);
	g_free(file);

	file = g_strconcat(g_get_home_dir(),"/.ishzilla/.ishzilla-uri",NULL);
	unlink(file);
	g_free(file);

	return;
}


void ishzilla_usr1_signal(gint sig)
{
	FILE *fd;
	gchar *file;
	gchar *uri;
	IshzillaWindow *window;
	gchar line[256];

	file = g_strconcat(g_get_home_dir(),"/.ishzilla/.ishzilla-uri",NULL);
	fd = fopen(file,"r");

	if (fd == NULL) {
g_warning("Could not open uri file!");
		g_free(file);
		window = ishzilla_window_new();
		ishzilla_app_add_window(app,window);

		return;
	}
	
	if (fgets(line,256,fd) == NULL) {
g_message("Could not read uri from uri file!");
		g_free(file);
		window = ishzilla_window_new();
		ishzilla_app_add_window(app,window);

		return;
	}

	fclose(fd);

	window = ishzilla_window_new_with_uri(line);
	ishzilla_app_add_window(app,window);

	unlink(file);

	g_free(file);

	return;
}

