/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "ishzilla-preferences.h"
#include "mozilla.h"


GConfClient *preferences_client = NULL;

static gchar * construct_real_key(const gchar *key)
{
	gchar *ret;

	ret = g_strconcat("/apps/ishzilla/",key,NULL);

	return ret;
}

static void check_first_run(void)
{
	gchar *s;
	gchar *b;

	s = g_strconcat(g_get_home_dir(),"/",".ishzilla",NULL);

	if (g_file_test(s,G_FILE_TEST_EXISTS) == FALSE) {
		mkdir(s,S_IRWXU);
	}

	b = g_strconcat(s,"/bookmarks.xml",NULL);

	if (g_file_test(b,G_FILE_TEST_EXISTS) == FALSE) {
		FILE *f = fopen(b,"w");

		fprintf(f,"<?xml version=\"1.0\"?>\n<izb version=\"1.0\">\n<bookmark title=\"Google\" uri=\"http://www.google.com\" />\n</izb>\n");
		fclose(f);
	}

	g_free(s);
	g_free(b);


	return;
}


void ishzilla_preferences_init(gint *argc,gchar ***argv)
{
	GSList *entries;
	GSList *l;

	if (preferences_client) {
		return;
	}

	preferences_client = gconf_client_get_default();

	check_first_run();

/*	entries = gconf_client_all_entries(preferences_client,"/apps/ishzilla",NULL);

	for (l = entries; l; l = l->next) {
		GConfEntry *entry = l->data;
		const gchar *key = gconf_entry_get_key(entry);
		GConfValue *value = gconf_entry_get_value(entry);
		GConfValueType type = value->type;
		const gchar *moz_key;

		moz_key = strrchr(key,'/');
		moz_key++;

		switch (type) {
			case GCONF_VALUE_STRING:
			{
				const gchar *s = gconf_value_get_string(value);

				mozilla_preferences_set_string(moz_key,s);

				break;
			}	
			case GCONF_VALUE_INT:
			{
				gint i = gconf_value_get_int(value);

				mozilla_preferences_set_int(moz_key,i);
	
				break;
			}
			case GCONF_VALUE_BOOL:
			{
				gboolean b = gconf_value_get_bool(value);

				mozilla_preferences_set_bool(moz_key,b);

				break;
			}
			default:
				g_assert_not_reached();
		}

	}
*/
	return;
}

gboolean ishzilla_preferences_get_bool(const gchar *key)
{
	gchar *real_key = construct_real_key(key);
	gboolean val = gconf_client_get_bool(preferences_client,real_key,NULL);

	g_free(real_key);

	return val;
}

void ishzilla_preferences_set_bool(const gchar *key,gboolean value)
{
	gchar *real_key = construct_real_key(key);

	gconf_client_set_bool(preferences_client,real_key,value,NULL);
	g_free(real_key);

	return;
}

gchar * ishzilla_preferences_get_string(const gchar *key)
{
	gchar *real_key = construct_real_key(key);
	gchar *val = gconf_client_get_string(preferences_client,real_key,NULL);

	g_free(real_key);

	return val;
}

void ishzilla_preferences_set_string(const gchar *key,const gchar *value)
{
	gchar *real_key = construct_real_key(key);

	gconf_client_set_string(preferences_client,real_key,value,NULL);
	g_free(real_key);

	return;
}


gint ishzilla_preferences_get_int(const gchar *key)
{
	gchar *real_key = construct_real_key(key);
	gint val = gconf_client_get_int(preferences_client,real_key,NULL);

	g_free(real_key);

	return val;
}

void ishzilla_preferences_set_int(const gchar *key,const gint value)
{
	gchar *real_key = construct_real_key(key);

	gconf_client_set_int(preferences_client,real_key,value,NULL);
	g_free(real_key);

	return;
}

void ishzilla_preferences_finish(void)
{
	if (preferences_client) {
		g_object_unref(preferences_client);
	}

	mozilla_preferences_finish();

	return;
}

