/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include "ishzilla.h"
#include "ishzilla-app.h"
#include "ishzilla-preferences.h"
#include "ishzilla-bookmark.h"
#include "ishzilla-bookmark-store.h"
#include "ishzilla-module.h"
#include "ishzilla-ipc.h"

IshzillaApp *app;
static GtkTreeModel *bookmark_store;

GList *modules;

static void app_quit(IshzillaApp *app)
{
	g_return_if_fail(app != NULL);
	g_return_if_fail(ISHZILLA_IS_APP(app));

	g_object_unref(app);

	gtk_main_quit();

	return;
}

gint main(gint argc,gchar **argv)
{
	IshzillaBookmark *bookmarks;
	gchar *s;
	guint i;
	gboolean no_ipc = FALSE;

	ishzilla_preferences_init(&argc,&argv);

	for (i = 0; i < argc; i++) {
		if (g_ascii_strcasecmp(argv[i],"--no-ipc") == 0) {
/* FIXME do it like gtk and remove the entry/decrement argc */
			argv[i][0] = '\0';
			no_ipc = TRUE;
		}
	}

	if (no_ipc == FALSE) {
		ishzilla_check_running(&argc,&argv);

		/* we returned from check_running, so we're the first! */
		ishzilla_set_running();
	     	signal(SIGUSR1,ishzilla_usr1_signal); /* connect the signal to launch new windows if running */
	}

	gtk_init(&argc,&argv);
	
	s = g_strconcat(g_get_home_dir(),"/",".ishzilla/bookmarks.xml",NULL);
	bookmarks = ishzilla_bookmark_new_from_file(s);
	bookmark_store = GTK_TREE_MODEL(ishzilla_bookmark_store_new(bookmarks));

	modules = ishzilla_module_list_new(ISHZILLA_PLUGIN_DIR);

	app = ishzilla_app_new(&argc,&argv);
	g_signal_connect(G_OBJECT(app),"_quit",G_CALLBACK(app_quit),NULL);

	gtk_main();

	gtk_moz_embed_pop_startup();

	ishzilla_end_running(); /* remove pid file */

	ishzilla_preferences_finish();

	bookmarks = ishzilla_bookmark_store_get_bookmarks(ISHZILLA_BOOKMARK_STORE(bookmark_store));
	ishzilla_bookmark_write(bookmarks,s);
	g_object_unref(bookmarks);
	g_object_unref(bookmark_store);

	ishzilla_module_list_free(modules);

	g_free(s);

	return 0;
}



GtkWidget * gtk_button_new_with_widget(GtkWidget *widget)
{
	GtkWidget *b = gtk_button_new();

	gtk_container_add(GTK_CONTAINER(b),widget);

	return b;
}

GtkWidget * gtk_button_new_from_stock_without_label(const gchar *stock_id)
{
	GtkWidget *i = gtk_image_new_from_stock(stock_id,GTK_ICON_SIZE_BUTTON);

	return gtk_button_new_with_widget(i);
}

GtkWidget * gtk_button_new_from_stock_with_label(const gchar *stock_id,const gchar *label)
{
	GtkWidget *align = gtk_alignment_new(0.5,0.5,0,0);
	GtkWidget *hbox = gtk_hbox_new(FALSE,2);
	GtkWidget *i = gtk_image_new_from_stock(stock_id,GTK_ICON_SIZE_BUTTON);
	GtkWidget *l = gtk_label_new_with_mnemonic(label);
	GtkWidget *b;

	gtk_misc_set_alignment(GTK_MISC(l),0.5,0.5);

	gtk_container_add(GTK_CONTAINER(align),hbox);
	gtk_box_pack_start(GTK_BOX(hbox),i,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),l,FALSE,FALSE,0);
	

	b = gtk_button_new_with_widget(align);
	gtk_label_set_mnemonic_widget(GTK_LABEL(l),b);

	return b;
}

void gtk_label_set_font(GtkLabel *label,const gchar *family)
{
	PangoAttrList *attrs;
	PangoAttribute *family_attr;

	attrs = pango_attr_list_new();

	family_attr = pango_attr_family_new(family);
	family_attr->start_index = 0;
	family_attr->end_index = G_MAXINT;
	pango_attr_list_insert(attrs,family_attr);

	gtk_label_set_use_underline(GTK_LABEL(label),FALSE);
	gtk_label_set_use_markup(GTK_LABEL(label),FALSE);
	gtk_label_set_attributes(GTK_LABEL(label),attrs);

	pango_attr_list_unref(attrs);

	return;
}
	
void gtk_window_set_up_geometry(GtkWindow *window)
{
 /* set it the minimum width & height so the window's reported size is accurate instead of relative to a false base width/height */
	GdkGeometry geom;

	geom.min_width = 0;
	geom.min_height = 0;

	gtk_window_set_geometry_hints(window,GTK_WIDGET(window),&geom,GDK_HINT_MIN_SIZE);

	return;
}
	
gchar * ish_strcasestr(const gchar *haystack,const gchar *needle)
{
	gchar *p;
	gchar *max;
	gint l;

	if (haystack && needle) {
		l = strlen(needle);
		max = (gchar *)haystack + strlen(haystack) - l;

		for (p = (gchar *)haystack; p <= max; p++) {
			if (g_ascii_strncasecmp(p,needle,l) == 0) {
				return p;
			}
		}
	}

	return NULL;
}

void ishzilla_quit(void)
{
	ishzilla_app_quit(app);

	return;
}

GtkTreeModel * ishzilla_get_bookmark_store(void)
{
	g_object_ref(bookmark_store);

	return bookmark_store;
}

