/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include "ishzilla-file-dialog.h"
#include "mozilla.h"

#include <nsCOMPtr.h>
#include <nsString.h>
#include <nsReadableUtils.h>
#include <nsIFileURL.h>
#include <nsIURI.h>
#include <nsIFileChannel.h>
#include <nsIChannel.h>
#include <nsIEmbeddingSiteWindow.h>
#include <nsIWindowWatcher.h>
#include <nsIWebBrowserChrome.h>
#include <nsNetCID.h>

#include <gtk/gtk.h>


NS_IMPL_ISUPPORTS1(IshzillaFileDialog,nsIFilePicker)


IshzillaFileDialog::IshzillaFileDialog()
{
	NS_INIT_ISUPPORTS();

	my_file = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
	my_directory = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
	my_directory->InitWithNativePath(nsDependentCString(g_get_home_dir()));

}

IshzillaFileDialog::~IshzillaFileDialog()
{

}


GtkWidget * IshzillaFileDialog::find_gtk_parent(nsIDOMWindow *window)
{
        nsresult result;

        nsCOMPtr<nsIWindowWatcher> wwatch (do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
        if (!wwatch) {
		return nsnull;
	}

 	nsCOMPtr<nsIDOMWindow> dom_window(window);
	if (!dom_window) {
		result = wwatch->GetActiveWindow(getter_AddRefs(dom_window));
		if (NS_FAILED(result) || !dom_window) {
			return nsnull;
		}
	}
	
        nsCOMPtr<nsIWebBrowserChrome> window_chrome;
        result = wwatch->GetChromeForWindow(dom_window,getter_AddRefs(window_chrome));
        if (NS_FAILED(result)) {
		return nsnull;
	}

        nsCOMPtr<nsIEmbeddingSiteWindow> site_window(do_QueryInterface(window_chrome,&result));
        if (NS_FAILED(result)) {
		return nsnull;
	}

        GtkWidget *mozembed;
        result = site_window->GetSiteWindow((void **)&mozembed);
        if (NS_FAILED(result)) {
		return nsnull;
	}

	return gtk_widget_get_toplevel(mozembed);
}

NS_METHOD IshzillaFileDialog::handle_file_picker_result(PRInt16 *retval)
{
	*retval = nsIFilePicker::returnCancel;
	nsresult rv;

	const char *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(my_file_selector));


	if (!filename || filename[0] == '\0') {
		return NS_ERROR_FAILURE;
	}

	const nsACString &filename_string = nsDependentCString(filename);
	my_file->InitWithNativePath(filename_string);

	if (my_mode == nsIFilePicker::modeGetFolder) {
		my_directory->InitWithNativePath(filename_string);
		my_default_string = "";
	} else {
		nsCOMPtr<nsIFile> directory;

		my_file->GetParent(getter_AddRefs(directory));
		my_directory = do_QueryInterface(directory);
		my_file->GetNativeLeafName(my_default_string);
	}

	*retval = nsIFilePicker::returnOK;

	return NS_OK;

}


/* *****************************
   nsIFilePicker stubs
   ***************************** */
NS_IMETHODIMP IshzillaFileDialog::Init(nsIDOMWindowInternal *parent,const PRUnichar *title,PRInt16 mode)
{
	my_parent = do_QueryInterface(parent);
	my_parent_widget = find_gtk_parent(my_parent);
	my_title = NS_ConvertUCS2toUTF8(title);
	my_mode = mode;
	
	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::AppendFilters(PRInt32 filter_mask)
{
	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::AppendFilter(const PRUnichar *title, const PRUnichar *filter)
{
	my_filter = NS_ConvertUCS2toUTF8(filter);
	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::GetFilterIndex(PRInt32 *filter_index)
{
	return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP IshzillaFileDialog::SetFilterIndex(PRInt32 filter_index)
{
	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::GetDefaultString(PRUnichar **default_string)
{
	guint bytes_read = 0;
	guint bytes_written = 0;
	gchar *utf8_default_string = g_filename_to_utf8(my_default_string.get(),-1,&bytes_read,&bytes_written,NULL);

	*default_string = ToNewUnicode(NS_ConvertUTF8toUCS2(utf8_default_string));
	g_free(utf8_default_string);

	return NS_OK;
}
NS_IMETHODIMP IshzillaFileDialog::SetDefaultString(const PRUnichar *default_string)
{
	if (default_string) {
		guint bytes_read = 0;
		guint bytes_written = 0;
		gchar *charstar_default_string = g_filename_from_utf8(NS_ConvertUCS2toUTF8(default_string).get(),-1,&bytes_read,&bytes_written,NULL);

		my_default_string = charstar_default_string;

		g_free(charstar_default_string);
	} else {
		my_default_string = "";
	}

	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::GetDefaultExtension(PRUnichar **default_extension)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP IshzillaFileDialog::SetDefaultExtension(const PRUnichar *default_extension)
{
    return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::GetDisplayDirectory(nsILocalFile **display_directory)
{
	NS_IF_ADDREF(*display_directory = my_directory);
	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::SetDisplayDirectory(nsILocalFile * display_directory)
{
	my_directory = display_directory;
	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::GetFile(nsILocalFile **file)
{
	NS_IF_ADDREF(*file = my_file);
	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::GetFileURL(nsIFileURL **file_url)
{
	nsCOMPtr<nsIFileURL> new_file_url = do_CreateInstance(NS_STANDARDURL_CONTRACTID);

	new_file_url->SetFile(my_file);
	NS_IF_ADDREF(*file_url = new_file_url);

	return NS_OK;
}

NS_IMETHODIMP IshzillaFileDialog::GetFiles(nsISimpleEnumerator **files)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP IshzillaFileDialog::Show(PRInt16 *retval)
{
	my_file_selector = gtk_file_selection_new(my_title.get());

	nsCAutoString filename;
	if(my_mode == nsIFilePicker::modeGetFolder) {
		filename.Assign("");
	} else {
		filename = my_default_string;
	}

	nsCAutoString dirname;
	my_directory->GetNativePath(dirname);

	nsCAutoString fullpath;
	fullpath.Assign(dirname + NS_LITERAL_CSTRING("/") + filename);

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(my_file_selector),fullpath.get());

	if (!my_filter.IsEmpty()) {
		gtk_file_selection_complete(GTK_FILE_SELECTION(my_file_selector),my_filter.get());
	}

	if (my_parent_widget) {
		gtk_window_set_transient_for(GTK_WINDOW(my_file_selector),GTK_WINDOW(my_parent_widget));
	}

	if (my_mode == nsIFilePicker::modeGetFolder) {
		gtk_widget_hide(GTK_FILE_SELECTION(my_file_selector)->file_list->parent);
	}

	gtk_window_set_modal(GTK_WINDOW(my_file_selector),TRUE);

	gint r = gtk_dialog_run(GTK_DIALOG(my_file_selector));
	
	if (r == GTK_RESPONSE_OK) {
		handle_file_picker_result(retval);
	} else {
		*retval = nsIFilePicker::returnCancel;
	}

	gtk_widget_destroy(my_file_selector);

	return NS_OK;
}

