/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include <mozilla-config.h>

#include <gtkmozembed.h>
#include <gtkmozembed_internal.h>

#include <nsCOMPtr.h>
#include <nsIInterfaceRequestorUtils.h>
#include <nsIDocShell.h>
#include <nsIDocShellTreeItem.h>
#include <nsIDocShellTreeOwner.h>
#include <nsIWebBrowser.h>
#include <nsIWebNavigation.h>
#include <nsIWebPageDescriptor.h>
#include <nsISHistory.h>
#include <nsIHistoryEntry.h>
#include <nsIContentViewer.h>
#include <nsIMarkupDocumentViewer.h>
#include <nsIClipboardCommands.h>
#include <nsIDocumentViewer.h>
#include <nsIPresContext.h>
#include <nsIPresShell.h>
#include <nsIDocument.h>
#include <nsIAtom.h>
#include <nsString.h>
#include <nsVoidArray.h>
#include <nsIURI.h>

#include <nsIStyleSheet.h>
#include <nsIStyleSet.h>

/* some taken from galeon 1.3.2, thanks guys */
static nsresult get_doc_shell(nsIWebBrowser *browser,nsIDocShell **doc_shell)
{
	nsCOMPtr<nsIDocShellTreeItem> browser_item;

	browser_item = do_QueryInterface(browser);
	if (!browser_item) {
		return NS_ERROR_FAILURE;
	}
      
        // get the owner for that item
        nsCOMPtr<nsIDocShellTreeOwner> tree_owner;
	browser_item->GetTreeOwner(getter_AddRefs(tree_owner));
	if (!tree_owner) {
		return NS_ERROR_FAILURE;
	}

        // get the primary content shell as an item
        nsCOMPtr<nsIDocShellTreeItem> content_item;
        tree_owner->GetPrimaryContentShell(getter_AddRefs(content_item));
	if (!content_item) {
		return NS_ERROR_FAILURE;
	}

        // QI that back to a docshell
        nsCOMPtr<nsIDocShell> doc_shell_ret;
	doc_shell_ret = do_QueryInterface(content_item);
	if (!doc_shell) {
		return NS_ERROR_FAILURE;
	}

	*doc_shell = doc_shell_ret.get();

        NS_IF_ADDREF(*doc_shell);
        
        return NS_OK;
}


static nsresult get_shistory(nsIWebBrowser *browser,nsISHistory **shistory)
{
	nsresult result;
	nsCOMPtr<nsIDocShell> doc_shell;

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return NS_ERROR_FAILURE;
	}

	nsCOMPtr<nsIWebNavigation> nav = do_QueryInterface(doc_shell,&result);
	if (!nav) {
		return NS_ERROR_FAILURE;
	}

	nsCOMPtr<nsISHistory> shistory_ret;
	result = nav->GetSessionHistory(getter_AddRefs(shistory_ret));
	if (!shistory_ret) {
		return NS_ERROR_FAILURE;
	}

	*shistory = shistory_ret.get();
	NS_IF_ADDREF (*shistory);

	return NS_OK;
}

extern "C" gboolean moz_embed_get_history_info(GtkMozEmbed *moz,gint *count,gint *index)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsISHistory> history;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		*count = -1;
		return FALSE;
	}

	result = get_shistory(browser,getter_AddRefs(history));
	if (NS_FAILED(result) || !history) {
		*count = -1;
		return FALSE;
	}

	history->GetCount((PRInt32*)count);
	history->GetIndex((PRInt32*)index);

	return TRUE;
}

extern "C" gboolean moz_embed_get_history_index_info(GtkMozEmbed *moz,gint index,gchar **url,gchar **title)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsISHistory> history;

	if (url == NULL && title == NULL) {
		return TRUE;
	}

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		*url = *title = NULL;
		return FALSE;
	}

	result = get_shistory(browser,getter_AddRefs(history));
	if (NS_FAILED(result) || !history) {
		*url = *title = NULL;
		return FALSE;
	}

	nsCOMPtr<nsIHistoryEntry> entry;
	result = history->GetEntryAtIndex((PRInt32)index,PR_FALSE,getter_AddRefs(entry));
	if (NS_FAILED(result) || !entry) {
		*url = *title = NULL;
		return FALSE;
	}

	PRUnichar *pruc_title;
	if (title) {
		result = entry->GetTitle(&pruc_title);
		if (NS_FAILED(result) || !pruc_title) {
			*url = *title = NULL;
			return FALSE;
		}
	}

	if (url) {
		nsCOMPtr<nsIURI> uri;
		nsCString cstr_url;
		result = entry->GetURI(getter_AddRefs(uri));
		if (NS_FAILED(result) || !uri) {
		*url = *title = NULL;
			return FALSE;
		}

		result = uri->GetSpec(cstr_url);
		if (NS_FAILED(result) || cstr_url.IsEmpty()) {
			*url = *title = NULL;
			return FALSE;
		}

		*url = g_strdup(cstr_url.get());
	}

	if (title) {
		*title = g_strdup(NS_ConvertUCS2toUTF8(pruc_title).get());
	}

	return TRUE;
}

extern "C" gboolean moz_embed_goto_history_index(GtkMozEmbed *moz,gint index)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsCOMPtr<nsIWebNavigation> nav = do_QueryInterface(doc_shell,&result);
	if (!nav) {
		return FALSE;
	}

	result = nav->GotoIndex(index);

	if (NS_FAILED(result)) {
		return FALSE;
	}

	return TRUE;
}

extern "C" gboolean moz_embed_set_zoom(GtkMozEmbed *moz,gfloat zoom)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsCOMPtr<nsIContentViewer> content_viewer;	
	result = doc_shell->GetContentViewer(getter_AddRefs(content_viewer));

	if (NS_FAILED(result) || !content_viewer) {
		return FALSE;
	}

	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(content_viewer,&result);
	if (NS_FAILED(result) || !mdv) {
		return FALSE;
	}

	result = mdv->SetTextZoom(zoom);

	if (NS_FAILED(result)) {
		return FALSE;
	}

	return TRUE;
}

extern "C" gboolean moz_embed_get_zoom(GtkMozEmbed *moz,gfloat *zoom)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsCOMPtr<nsIContentViewer> content_viewer;	
	result = doc_shell->GetContentViewer(getter_AddRefs(content_viewer));

	if (NS_FAILED(result) || !content_viewer) {
		return FALSE;
	}

	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(content_viewer,&result);
	if (NS_FAILED(result) || !mdv) {
		return FALSE;
	}

	result = mdv->GetTextZoom(zoom);

	if (NS_FAILED(result)) {
		return FALSE;
	}

	return TRUE;
}

extern "C" gboolean moz_embed_select_all(GtkMozEmbed *moz)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	nsCOMPtr<nsIClipboardCommands> clipboard(do_GetInterface(browser));

	result = clipboard->SelectAll();

	if (NS_FAILED(result)) {
		return FALSE;
	}

	return TRUE;
}

static nsresult get_pres_shell(nsIDocShell *doc_shell,nsIPresShell **pres_shell)
{
	nsresult result;

	nsCOMPtr<nsIContentViewer> content_viewer;	
	result = doc_shell->GetContentViewer(getter_AddRefs(content_viewer));
	if (NS_FAILED(result) || !content_viewer) {
		return FALSE;
	}

	nsCOMPtr<nsIDocumentViewer> document_viewer;
	document_viewer = do_QueryInterface(content_viewer,&result);
	if (NS_FAILED(result) || !document_viewer) {
		return FALSE;
	}

	nsIPresContext *context;
	document_viewer->GetPresContext(context);

	nsIPresShell *shell;
	context->GetShell(&shell);
	NS_RELEASE(context);

	*pres_shell = shell;

        return NS_OK;
}

extern "C" gboolean moz_embed_set_style(GtkMozEmbed *moz,const gchar *style)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsIPresShell *shell = nsnull;

	result = get_pres_shell(doc_shell,&shell);
	if (NS_FAILED(result) || !shell) {
		return FALSE;
	}


	nsString string;

	string.AssignWithConversion(style);

/* God Damned Fucking Stupid Son of a Bitch Cockmongers at mozilla */
//	result = shell->SelectAlternateStyleSheet(string);

	PRInt32 count = 0;
	nsIDocument *document;
	nsIStyleSet *style_set;

	shell->GetDocument(&document);
	shell->GetStyleSet(&style_set);

	document->GetNumberOfStyleSheets(PR_FALSE,&count);

	PRInt32 i;

	NS_NAMED_LITERAL_STRING(text_html,"text/html");

#warning need to somehow monitor the document or something for changes in stylesheet (from javascript, stupid something)
	for (i = 0; i < count; i++) {
		nsCOMPtr<nsIStyleSheet> sheet;
		PRBool complete;

		document->GetStyleSheetAt(i,PR_FALSE,getter_AddRefs(sheet));

		sheet->GetComplete(complete);
		if (complete) {
			nsAutoString type;

			sheet->GetType(type);
			if (!type.Equals(text_html)) {
				nsAutoString title;

				sheet->GetTitle(title);
				if (!title.IsEmpty() && title.Equals(string)) {
					sheet->SetEnabled(PR_TRUE);
					document->SetStyleSheetApplicableState(sheet,PR_TRUE);
/* God Damned Fucking Stupid Son of a Bitch Cockmongers at mozilla */
//					style_set->AddDocStyleSheet(sheet, document);
				} else {
					sheet->SetEnabled(PR_FALSE);
					document->SetStyleSheetApplicableState(sheet,PR_FALSE);
/* God Damned Fucking Stupid Son of a Bitch Cockmongers at mozilla */
//					style_set->RemoveDocStyleSheet(sheet);
				}
			}
		}
	}

	(shell);

	if (NS_FAILED(result)) {
		return FALSE;
	}

	return TRUE;
}

extern "C" gboolean moz_embed_get_style_list(GtkMozEmbed *moz,gchar ***list)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsIPresShell *shell = nsnull;

	result = get_pres_shell(doc_shell,&shell);
	if (NS_FAILED(result) || !shell) {
		return FALSE;
	}

	nsStringArray titles;
	
	result = shell->ListAlternateStyleSheets(titles);
	if (NS_FAILED(result)) {
g_print("Did not get any alternates\n");
		NS_RELEASE(shell);

		return FALSE;
	}

	PRInt32 count = titles.Count();

	gchar **a = g_new0(gchar *,count + 1);

	PRInt32 i;
	for (i = 0; i < count; i++) {
		nsAutoString title;

		titles.StringAt(i,title);

		a[i] = g_strdup(NS_ConvertUCS2toUTF8(title).get());
	}

	*list = a;

	NS_RELEASE(shell);

	return TRUE;
}

extern "C" gboolean moz_embed_get_current_style(GtkMozEmbed *moz,gchar **style)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsIPresShell *shell = nsnull;

	result = get_pres_shell(doc_shell,&shell);
	if (NS_FAILED(result) || !shell) {
		return FALSE;
	}

	nsAutoString current;
	shell->GetActiveAlternateStyleSheet(current);

	*style = g_strdup(NS_LossyConvertUCS2toASCII(current).get());

	NS_RELEASE(shell);

	return TRUE;
}

extern "C" gboolean moz_embed_get_default_style(GtkMozEmbed *moz,gchar **style)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsIPresShell *shell = nsnull;

	result = get_pres_shell(doc_shell,&shell);
	if (NS_FAILED(result) || !shell) {
		return FALSE;
	}

        nsAutoString  def;
        nsCOMPtr<nsIDocument> document;

	result = shell->GetDocument(getter_AddRefs(document));
	if (NS_FAILED(result) || !document) {
		NS_RELEASE(shell);
		return FALSE;
	}

	nsIAtom* atom = NS_NewAtom("default-style");
	document->GetHeaderData(atom,def);
	NS_RELEASE(atom);

	*style = g_strdup(NS_LossyConvertUCS2toASCII(def).get());

	NS_RELEASE(shell);


	return TRUE;
}

