/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include <mozilla-config.h>

#include "pdm.h"

#include <gtkmozembed.h>
#include <gtkmozembed_internal.h>

#include <nsIComponentRegistrar.h>
#include <nsIGenericFactory.h> // generic factory constructor macro
#include <nsIServiceManager.h> // do_GetService
#include <nsIServiceManagerUtils.h>
#include <nsIInterfaceRequestor.h> // do_GetInterface
#include <nsIInterfaceRequestorUtils.h>
#include <nsIPref.h> // pref shit
#include <nsIPrefBranch.h> // pref shit
#include <nsCOMPtr.h> // everything
#include <nsIDocument.h>
#include <nsIDOMDocument.h>
#include <nsIDocument.h>
#include <nsIDOMMouseEvent.h>
#include <nsIDOMEventTarget.h>
#include <nsIDOMHTMLElement.h>
#include <nsIDOMNamedNodeMap.h>
#include <nsIDOMNSHTMLElement.h>  
#include <nsIDOMKeyEvent.h>
#include <nsIDOMNSDocument.h>
#include <nsIDOMNSEvent.h>
#include <nsString.h> // string
#include <nsReadableUtils.h> // ToNewCString
#include <nsIURI.h>
#include <nsIWebBrowserFind.h> // Find stuff
#include <nsICookieManager.h>
#include <nsIPasswordManager.h>
#include <nsIPassword.h>
#include <nsICookie.h>
#include <nsCCookieManager.h>
#include <nsCPasswordManager.h>

#include "ishzilla-prompt-service.h"
#include "ishzilla-progress-dialog.h"
#include "ishzilla-file-dialog.h"
#include "ishzilla-content-handler.h"

#include <time.h>

#define MOZILLA_CONTEXT_NONE 0
#define MOZILLA_CONTEXT_DEFAULT  1
#define MOZILLA_CONTEXT_LINK     2
#define MOZILLA_CONTEXT_IMAGE    4
#define MOZILLA_CONTEXT_DOCUMENT 8
#define MOZILLA_CONTEXT_INPUT    64
#define MOZILLA_CONTEXT_OTHER    128
#define MOZILLA_CONTEXT_XUL      256

#define MOZILLA_MODIFIER_ALT 1
#define MOZILLA_MODIFIER_CTRL 2
#define MOZILLA_MODIFIER_SHIFT 4
#define MOZILLA_MODIFIER_META 8

NS_GENERIC_FACTORY_CONSTRUCTOR(IshzillaPromptService)
NS_GENERIC_FACTORY_CONSTRUCTOR(IshzillaProgressDialog)
NS_GENERIC_FACTORY_CONSTRUCTOR(IshzillaFileDialog)
NS_GENERIC_FACTORY_CONSTRUCTOR(IshzillaContentHandler)

static const nsModuleComponentInfo components[] = {
	{
		I_PROMPT_SERVICE_CLASSNAME,
		I_PROMPT_SERVICE_CID,
		I_PROMPT_SERVICE_CONTRACTID,
		IshzillaPromptServiceConstructor
	},
	{
		I_PROGRESS_DIALOG_CLASSNAME,
		I_PROGRESS_DIALOG_CID,
		I_PROGRESS_DIALOG_CONTRACTID,
		IshzillaProgressDialogConstructor
	},
	{
		I_PROGRESS_DIALOG_CLASSNAME,
		I_PROGRESS_DIALOG_CID,
		NS_DOWNLOAD_CONTRACTID,
		IshzillaProgressDialogConstructor
	},
	{
		I_FILE_DIALOG_CLASSNAME,
		I_FILE_DIALOG_CID,
		I_FILE_DIALOG_CONTRACTID,
		IshzillaFileDialogConstructor
	},
	{
		I_CONTENT_HANDLER_CLASSNAME,
		I_CONTENT_HANDLER_CID,
		NS_IHELPERAPPLAUNCHERDLG_CONTRACTID,
		IshzillaContentHandlerConstructor
	}
};

extern "C" gboolean mozilla_find(gpointer moz_embed,const gchar *text,gint direction,gboolean wrap,gboolean match_case)
{
	nsCOMPtr<nsIWebBrowser> browser;
	nsresult r;
	PRBool did_find;

	gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(moz_embed),getter_AddRefs(browser));

	if (!browser) {
		g_warning("Could not get nsIWebBrowser from GtkMozEmbed");
		return FALSE;
	}

	nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface(browser));
	
	finder->SetSearchString((NS_ConvertUTF8toUCS2(text)).get());
	finder->SetFindBackwards((direction >= 1 ? PR_FALSE : PR_TRUE));
	finder->SetWrapFind((wrap == TRUE ? PR_TRUE : PR_FALSE));
	finder->SetEntireWord(PR_FALSE);
	finder->SetMatchCase((match_case == TRUE ? PR_TRUE : PR_FALSE));
	finder->SetSearchFrames(PR_TRUE);

	r = finder->FindNext(&did_find);

	if (!NS_SUCCEEDED(r)) {
		g_warning("no success??\n");
		return FALSE;
	}

	if (did_find == PR_TRUE) {
		return TRUE;
	}

	return FALSE;
}

static gchar * mozilla_dom_node_get_attribute(nsIDOMNode *node,gchar *attribute)
{
	nsresult result;
	nsCOMPtr<nsIDOMNamedNodeMap> attributes;

	result = node->GetAttributes(getter_AddRefs (attributes));
	if (!NS_SUCCEEDED(result) || !attributes) {
		return NULL;
	}

	nsAutoString attr; 

	attr.AssignWithConversion (attribute);
                                        
	nsCOMPtr<nsIDOMNode> attr_node;
	result = attributes->GetNamedItem(attr,getter_AddRefs(attr_node));
	if (!NS_SUCCEEDED(result) || !attr_node) {
		return NULL;
	}

	nsAutoString node_value;
			
	result = attr_node->GetNodeValue(node_value);
	if (!NS_SUCCEEDED(result)) {
		return NULL;
	}
                                                
	gchar *cstr = ToNewCString(node_value);
	gchar *val = g_strdup (cstr);
			
	nsMemory::Free (cstr);
	
	return val;
}




extern "C" gint mozilla_dom_event_get_button(gpointer dom_event)
{
	PRUint16 button;

	((nsIDOMMouseEvent *)dom_event)->GetButton(&button);

	return (gint)button;
}

extern "C" glong mozilla_dom_event_get_context_menu_type(gpointer dom_event,gchar **img,gchar **uri,gchar **link_text)
{
	nsresult result;
	nsIDOMMouseEvent *mouse_event = (nsIDOMMouseEvent *)dom_event;
	nsCOMPtr<nsIDOMEventTarget> original_target;
	nsCOMPtr<nsIDOMNSEvent> event = do_QueryInterface(mouse_event);

	if (!event) {
		return MOZILLA_CONTEXT_NONE;
	}

     	result = event->GetOriginalTarget(getter_AddRefs(original_target));
	if (NS_FAILED(result) || !original_target) {
		return MOZILLA_CONTEXT_NONE;
	}

	nsCOMPtr<nsIDOMNode> original_node = do_QueryInterface(original_target);
	if (!original_node) {
		return MOZILLA_CONTEXT_NONE;
	}

	nsString node_name;
	original_node->GetNodeName(node_name);

	if (node_name.EqualsWithConversion("xul:thumb") ||
		node_name.EqualsWithConversion("xul:scrollbarbutton") ||
		node_name.EqualsWithConversion("xul:vbox") ||
		node_name.EqualsWithConversion("xul:spacer") ||
		node_name.EqualsWithConversion("xul:slider")) {

		return MOZILLA_CONTEXT_NONE;
	}

	nsCOMPtr<nsIDOMEventTarget> event_target;
	result = mouse_event->GetTarget(getter_AddRefs(event_target));
	if (NS_FAILED(result) || !event_target) {
		return MOZILLA_CONTEXT_NONE;
	}

	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(event_target);
	if (!node) {
		return MOZILLA_CONTEXT_NONE;
     	}

     	nsCOMPtr<nsIDOMDocument> dom_document;
     	result = node->GetOwnerDocument(getter_AddRefs(dom_document));
	if (!NS_SUCCEEDED(result) || !dom_document) {
		return MOZILLA_CONTEXT_NONE;
	}
     
     	nsCOMPtr<nsIDocument> document = do_QueryInterface(dom_document);
	if (!document) {
		return MOZILLA_CONTEXT_NONE;
	}
	
     	nsCOMPtr<nsIDOMNSDocument> ns_document = do_QueryInterface(dom_document);
	if (!ns_document) {
		return MOZILLA_CONTEXT_NONE;
	}

     	nsIURI *base_uri;
     	result = document->GetBaseURL(base_uri);
	if (NS_FAILED(result) || !base_uri) {
		return MOZILLA_CONTEXT_NONE;
	}

     	nsString mime;
     	ns_document->GetContentType(mime);
     	if (mime.EqualsWithConversion("text/xul")) {
		return MOZILLA_CONTEXT_NONE;
	}

	PRUint32 flags = MOZILLA_CONTEXT_NONE;
	nsCOMPtr<nsIDOMHTMLElement> element;

	do {
		PRUint16 type;
		node->GetNodeType(&type);

		element = do_QueryInterface(node);
		if (element) {
			nsAutoString tag;

			element->GetTagName(tag);

			if (tag.EqualsWithConversion("input",PR_TRUE)) {
				flags |= MOZILLA_CONTEXT_INPUT;
			} else if (tag.EqualsWithConversion("img",PR_TRUE)) {
				flags |= MOZILLA_CONTEXT_IMAGE;

				gchar *src = mozilla_dom_node_get_attribute(node,"src");
				if (!src) {
					return MOZILLA_CONTEXT_NONE;
				}

				nsAutoString src_string;
				src_string.AssignWithConversion(src);
			
				nsCString src_cstring;
				src_cstring.AssignWithConversion(src_string);

				nsCString img_string;
				result = base_uri->Resolve(src_cstring,img_string);

				g_free(src);

				*img = ToNewCString(img_string);

				if (!img) {
					return MOZILLA_CONTEXT_NONE;
				}
			} else {
				flags |= MOZILLA_CONTEXT_OTHER;
			}

			nsCOMPtr<nsIDOMNamedNodeMap> attributes;
			node->GetAttributes(getter_AddRefs(attributes));
			if (attributes) {
				nsCOMPtr<nsIDOMNode> href_node;
				nsAutoString href;

			     	href.AssignWithConversion("href");
				attributes->GetNamedItem(href, getter_AddRefs(href_node));
				if (href_node) {
					flags |= MOZILLA_CONTEXT_LINK;
				     	nsAutoString linkhtml;
				     	nsCOMPtr<nsIDOMNSHTMLElement> ns_element;
				     	
				     	ns_element = do_QueryInterface (element);				      
				     	
				     	if (ns_element) {
						result = ns_element->GetInnerHTML(linkhtml);
					     	if (NS_SUCCEEDED(result) && ToNewCString(linkhtml)) 
					       		*link_text = g_strdup(ToNewCString(linkhtml));
					}

					gchar *href =  mozilla_dom_node_get_attribute(node,"href");
					if (!href) {
						return MOZILLA_CONTEXT_NONE;
					}

				     	nsAutoString hrefa;
				     	hrefa.AssignWithConversion(href);
				     
				     	nsCString hrefc,linkc;
				     	hrefc.AssignWithConversion(hrefa);

				     	result = base_uri->Resolve(hrefc,linkc);
				     	g_free(href);

					*uri = ToNewCString(linkc);
				     
					if (!uri) {
						return MOZILLA_CONTEXT_NONE;
					}

					break;
				}
			}
		}
		nsCOMPtr<nsIDOMNode> parentNode;
		node->GetParentNode(getter_AddRefs(parentNode));
		
		if (!parentNode) {
			node = nsnull;
			flags |= MOZILLA_CONTEXT_DOCUMENT;
			break;
		}
		node = parentNode;

	} while (node);

	return flags;
}


extern "C" glong mozilla_dom_event_get_key(gpointer dom_event,glong *modifier)
{
	PRUint32 key;
	nsIDOMKeyEvent *key_event = (nsIDOMKeyEvent *)dom_event;
	PRBool mod_key;

	*modifier = 0;

	key_event->GetCharCode(&key);

	if (key == 0) {
		key_event->GetKeyCode(&key);
	}

	key_event->GetAltKey(&mod_key);
	if (mod_key) {
		*modifier |= MOZILLA_MODIFIER_ALT;
	}
	
	key_event->GetShiftKey(&mod_key);
	if (mod_key) {
		*modifier |= MOZILLA_MODIFIER_SHIFT;
	}

	key_event->GetCtrlKey(&mod_key);
	if (mod_key) {
		*modifier |= MOZILLA_MODIFIER_CTRL;
	}

	key_event->GetMetaKey(&mod_key);
	if (mod_key) {
		*modifier |= MOZILLA_MODIFIER_META;
	}

	return key;
}
		

extern "C" gboolean mozilla_preferences_set_string(const gchar *key,const gchar *value)
{
        nsCOMPtr<nsIPrefService> pref_service = do_GetService(NS_PREFSERVICE_CONTRACTID);
        nsCOMPtr<nsIPrefBranch> pref;

        pref_service->GetBranch("",getter_AddRefs(pref));

	if (pref) {
		nsresult result = pref->SetCharPref(key,value);

		return (NS_SUCCEEDED(result) ? TRUE : FALSE);
	}

	return FALSE;
}

extern "C" gchar * mozilla_preferences_get_string(const gchar *key)
{
        nsCOMPtr<nsIPrefService> pref_service = do_GetService(NS_PREFSERVICE_CONTRACTID);
        nsCOMPtr<nsIPrefBranch> pref;
	gchar *ret = NULL;

        pref_service->GetBranch("",getter_AddRefs(pref));

	if (pref) {
		gchar *r;

		nsresult result = pref->GetCharPref(key,&r);

		ret = g_strdup(r);
		nsMemory::Free(r);
	}

	return ret;
}

extern "C" gboolean mozilla_preferences_set_bool(const gchar *key,gboolean value)
{
        nsCOMPtr<nsIPrefService> pref_service = do_GetService(NS_PREFSERVICE_CONTRACTID);
        nsCOMPtr<nsIPrefBranch> pref;

        pref_service->GetBranch("",getter_AddRefs(pref));

	if (pref) {
		nsresult result = pref->SetBoolPref(key,(value ? PR_TRUE : PR_FALSE));
		return (NS_SUCCEEDED(result) ? TRUE : FALSE);
	}

	return FALSE;
}

extern "C" gboolean mozilla_preferences_get_bool(const gchar *key)
{
        nsCOMPtr<nsIPrefService> pref_service = do_GetService(NS_PREFSERVICE_CONTRACTID);
        nsCOMPtr<nsIPrefBranch> pref;
	gboolean ret = FALSE;

        pref_service->GetBranch("",getter_AddRefs(pref));

	if (pref) {
		PRBool r;

		nsresult result = pref->GetBoolPref(key,&r);

		ret = !(r == PR_FALSE);
	}

	return ret;
}


extern "C" gboolean mozilla_preferences_set_int(const gchar *key,gint value)
{
        nsCOMPtr<nsIPrefService> pref_service = do_GetService(NS_PREFSERVICE_CONTRACTID);
        nsCOMPtr<nsIPrefBranch> pref;

        pref_service->GetBranch("",getter_AddRefs(pref));

	if (pref) {
		nsresult result = pref->SetIntPref(key,value);
		return (NS_SUCCEEDED(result) ? TRUE : FALSE);
	}

	return FALSE;
}

#warning error handling is.. nonexistant, damn
extern "C" gint mozilla_preferences_get_int(const gchar *key)
{
        nsCOMPtr<nsIPrefService> pref_service = do_GetService(NS_PREFSERVICE_CONTRACTID);
        nsCOMPtr<nsIPrefBranch> pref;
	gint ret = 0;

        pref_service->GetBranch("",getter_AddRefs(pref));

	if (pref) {
		nsresult result = pref->GetIntPref(key,(PRInt32*)&ret);
	}

	return ret;
}


extern "C" gboolean mozilla_preferences_finish(void)
{
	nsCOMPtr<nsIPrefService> pref_service = do_GetService(NS_PREFSERVICE_CONTRACTID);

	nsresult result = pref_service->SavePrefFile(nsnull);
	return (NS_SUCCEEDED(result) ? TRUE : FALSE);
}

extern "C" gboolean mozilla_get_cookie_list(GList **cookies)
{
	nsresult result;
	nsCOMPtr<nsICookieManager> cookie_manager = do_CreateInstance(NS_COOKIEMANAGER_CONTRACTID);
	nsCOMPtr<nsISimpleEnumerator> i;

	*cookies = NULL;

	result = cookie_manager->GetEnumerator(getter_AddRefs(i));
	if (NS_FAILED(result)) {
		return FALSE;
	}

	PRBool r;

	for (i->HasMoreElements(&r); r == PR_TRUE; i->HasMoreElements(&r)) {
		Cookie *c;
		nsCOMPtr<nsICookie> cookie;

		result = i->GetNext(getter_AddRefs(cookie));
		if (NS_FAILED(result)) {
			return FALSE;
		}

		c = g_new0(Cookie,1);

		nsCAutoString s;

		cookie->GetHost(s);
		c->properties[COOKIE_SITE] = g_strdup(s.get());
		cookie->GetName(s);
		c->properties[COOKIE_NAME] = g_strdup(s.get());
		cookie->GetValue(s);
		c->properties[COOKIE_VALUE] = g_strdup(s.get());
		cookie->GetPath(s);
		c->properties[COOKIE_PATH] = g_strdup(s.get());
	
		PRBool secure;
		cookie->GetIsSecure(&secure);
		if (secure == PR_TRUE) {
			c->properties[COOKIE_SECURE] = g_strdup("Yes");
		} else {
			c->properties[COOKIE_SECURE] = g_strdup("No");
		}

		PRUint64 expires;
		cookie->GetExpires(&expires);
		if (expires == 0) {
			c->properties[COOKIE_EXPIRES] = g_strdup("End of current session");
		} else {
			c->properties[COOKIE_EXPIRES] = g_strdup_printf("%s",ctime((const time_t *)&expires));
			c->properties[COOKIE_EXPIRES][strlen(c->properties[COOKIE_EXPIRES])-1] = '\0';
		}

		*cookies = g_list_prepend(*cookies,c);
	}

	return TRUE;
}

extern "C" gboolean mozilla_remove_cookie(Cookie *cookie)
{
	nsresult result;
	nsCOMPtr<nsICookieManager> cookie_manager = do_CreateInstance(NS_COOKIEMANAGER_CONTRACTID);
	guint i;

	result = cookie_manager->Remove(NS_LITERAL_CSTRING(cookie->properties[COOKIE_SITE]),NS_LITERAL_CSTRING(cookie->properties[COOKIE_NAME]),NS_LITERAL_CSTRING(cookie->properties[COOKIE_PATH]),PR_FALSE);

#warning this doesnt actually... work?

	if (NS_FAILED(result)) {
		return FALSE;
	}	

	for (i = 0; i < 7; i++) {
		g_free(cookie->properties[i]);
	}

	g_free(cookie);	

	return TRUE;
}

extern "C" gboolean mozilla_get_password_list(GList **passwords)
{
	nsresult result;
	nsCOMPtr<nsIPasswordManager> password_manager = do_CreateInstance(NS_PASSWORDMANAGER_CONTRACTID);
	nsCOMPtr<nsISimpleEnumerator> i;

	*passwords = NULL;

	result = password_manager->GetEnumerator(getter_AddRefs(i));
	if (NS_FAILED(result)) {
		return FALSE;
	}

	PRBool r;

	for (i->HasMoreElements(&r); r == PR_TRUE; i->HasMoreElements(&r)) {
		Password *p;
		nsCOMPtr<nsIPassword> password;

		result = i->GetNext(getter_AddRefs(password));
		if (NS_FAILED(result)) {
			return FALSE;
		}

		p = g_new0(Password,1);

		nsCAutoString s;

		password->GetHost(s);
		p->properties[PASSWORD_SITE] = g_strdup(s.get());

		nsAutoString u;
		password->GetUser(u);
		p->properties[PASSWORD_USER] = g_strdup(NS_ConvertUCS2toUTF8(u).get());

		*passwords = g_list_prepend(*passwords,p);
	}

	return TRUE;
}

extern "C" gboolean mozilla_remove_password(Password *password)
{
	nsresult result;
	nsCOMPtr<nsIPasswordManager> password_manager = do_CreateInstance(NS_PASSWORDMANAGER_CONTRACTID);
	guint i;

	result = password_manager->RemoveUser(NS_LITERAL_CSTRING(password->properties[PASSWORD_SITE]),NS_ConvertUTF8toUCS2(nsDependentCString(password->properties[PASSWORD_USER])));

	if (NS_FAILED(result)) {
		return FALSE;
	}	

	for (i = 0; i < 2; i++) {
		g_free(password->properties[i]);
	}

	g_free(password);	

	return TRUE;
}

extern "C" gboolean mozilla_init(void)
{
	gboolean r = TRUE;
	nsresult result;
	nsCOMPtr<nsIComponentRegistrar> cr;

	mozilla_preferences_set_bool("accessibility.typeaheadfind",FALSE);
	mozilla_preferences_set_bool("browser.enable_automatic_image_resizing",FALSE);
	mozilla_preferences_set_bool("browser.related.enabled",FALSE);
	mozilla_preferences_set_bool("general.smoothScroll",TRUE);
	mozilla_preferences_set_bool("mozilla.widget.raise-on-setfocus",FALSE);
	mozilla_preferences_set_bool("network.dir.generate_html",TRUE);
	mozilla_preferences_set_bool("security.warn_entering_secure",FALSE);
	mozilla_preferences_set_bool("security.warn_leaving_secure",FALSE);
	mozilla_preferences_set_bool("security.warn_submit_insecure",FALSE);
	mozilla_preferences_set_bool("security.checkloaduri",FALSE);
	mozilla_preferences_set_bool("view_source.wrap_long_lines",TRUE);
	mozilla_preferences_set_bool("xpinstall.enabled",FALSE);

	result = NS_GetComponentRegistrar(getter_AddRefs(cr));
	NS_ENSURE_SUCCESS(result,result);

	for (gint i = 0; i < G_N_ELEMENTS(components); i++) {
		nsCOMPtr<nsIGenericFactory> factory;
	
		result = NS_NewGenericFactory(getter_AddRefs(factory),&(components[i]));
		if (NS_FAILED(result)) {
			r = FALSE;
			continue;
		}

		result = cr->RegisterFactory(components[i].mCID,components[i].mDescription,components[i].mContractID,factory);

		if (NS_FAILED(result)) {
			r = FALSE;
		}
	}

	return r;
}	

