/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include <stdio.h>
#include <string.h>

#include "ishzilla-module.h"
#include "ishzilla-preferences.h"

static GConfClient *prefs = NULL;

static gboolean gtkdoc_open_uri(IshzillaModule *module,GtkMozEmbed *moz,const gchar *uri);
static void gtkdoc_close(IshzillaModule *module);


static IshzillaModuleFunctions functions = {
	gtkdoc_open_uri,
	gtkdoc_close
};

static GHashTable *hash = NULL;

static guint gtkdoc_string_hash(const gchar *key)
{
	guint hash_key = 0;
	register int  i = 0;
	register char c;
	gchar *string = strchr(key,'-');
	
	if (string == NULL) {
		string = (gchar *)key;
	}

	do {
		c = string[i];
		if (c == '\0') {
			break;
		}

		hash_key += (((guint) c) << i);

		++i ;
	} while( i < ((sizeof (guint)-sizeof (char)) << 3) );

	return hash_key % 10000;
}
	
static gboolean gtkdoc_string_equal(const gchar *a,const gchar *b)
{
	if (g_ascii_strcasecmp(a,b) == 0) {
		return TRUE;
	}

	return FALSE;
}

static gchar * gtkdoc_get_key(gchar *line,const gchar *key)
{
	gchar *key_in_line = strstr(line,key);
	gchar *quote = strchr(key_in_line,'"');
	gchar *value = quote + 1;
	gchar *end_quote = strchr(value,'"');
	guint length = end_quote-value;

	return g_strndup(value,length);
}

gboolean init(IshzillaModule *module)
{
	gchar *doc_dir;
	GDir *dir;
	const gchar *file;

	prefs = gconf_client_get_default();
	
	doc_dir = gconf_client_get_string(prefs,"/apps/ishzilla/gtkdoc_doc_dir",NULL);

	if (doc_dir == NULL || doc_dir[0] == '\0') {
		doc_dir = g_strdup("/usr/share/gtk-doc/html");
	}


	module->functions = &functions;
	ishzilla_module_set_name(module,"Gtk Doc Search");
	ishzilla_module_set_author(module,"Charles Schmidt");
	ishzilla_module_set_author_email(module,"cschmidt2@emich.edu");
	ishzilla_module_add_preference(module,ISHZILLA_MODULE_PREFERENCE_TYPE_PATH,"_Path to Gtk Doc files","gtkdoc_doc_dir");

	hash = g_hash_table_new_full((GHashFunc)gtkdoc_string_hash,(GEqualFunc)gtkdoc_string_equal,(GDestroyNotify)g_free,(GDestroyNotify)g_free);
	
	dir = g_dir_open(doc_dir,0,NULL);
#warning check error on dir

	file = g_dir_read_name(dir);
	while (file) {
		gchar *index = g_strconcat(doc_dir,"/",file,"/index.sgml",NULL);
		gchar line[256];
		FILE *f;

		f = fopen(index,"r");
		if (f == NULL) {
			g_free(index);
			file = g_dir_read_name(dir);

			continue;
		}

		while (fgets(line,256,f)) {
			gchar *id = gtkdoc_get_key(line," id");
			gchar *href = gtkdoc_get_key(line," href");
						
			g_hash_table_insert(hash,id,href);
/* hash table will free id and href upon destruction */
		}

		fclose(f);
		g_free(index);

		file = g_dir_read_name(dir);
	}
	
	g_dir_close(dir);
	g_free(doc_dir);

	return TRUE;
}


static gboolean gtkdoc_open_uri(IshzillaModule *module,GtkMozEmbed *moz,const gchar *uri)
{
	if (g_ascii_strncasecmp(uri,"gtkdoc://",9) == 0) {
		const gchar *value;

		value = g_hash_table_lookup(hash,uri+9);

		if (value) {
			gchar *doc_dir;
			gchar *file;

			doc_dir = gconf_client_get_string(prefs,"/apps/ishzilla/gtkdoc_doc_dir",NULL);

			if (doc_dir == NULL || doc_dir[0] == '\0') {
				doc_dir = g_strdup("/usr/share/gtk-doc/html");
			}

			file = g_strconcat("file://",doc_dir,"/",value,NULL); /* cant forget the damn file://, stupid mozilla */

			gtk_moz_embed_load_url(moz,file);
		
			g_free(file);
			g_free(doc_dir);

			return TRUE;
		}
	}

	return FALSE;
}

static void gtkdoc_close(IshzillaModule *module)
{
	if (prefs) {
		g_object_unref(prefs);
	}

	g_hash_table_destroy(hash);
	return;
}


