/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include "ishzilla-module.h"
#include "ishzilla-preferences.h"

static gboolean rman_open_uri(IshzillaModule *module,GtkMozEmbed *moz,const gchar *uri);
static void rman_close(IshzillaModule *module);


static IshzillaModuleFunctions functions = {
	rman_open_uri,
	rman_close
};

static gchar *tmp_file;

gboolean init(IshzillaModule *module)
{
	pid_t pid;

	module->functions = &functions;
	ishzilla_module_set_name(module,"Man Page Reader");
	ishzilla_module_set_author(module,"Charles Schmidt");
	ishzilla_module_set_author_email(module,"cschmidt2@emich.edu");

	pid = getpid();
	tmp_file = g_strdup_printf("/tmp/ishzilla-man-%d",pid);

	return TRUE;
}


static gboolean rman_open_uri(IshzillaModule *module,GtkMozEmbed *moz,const gchar *uri)
{
	if (g_ascii_strncasecmp(uri,"man://",6) == 0) {
		const gchar *value;

		value = uri + 6;

		if (value[0] == '\0') {
#warning create an index to view all man pages
			return TRUE;
		} else {
			gchar *argv[] = {"superman",NULL,NULL,NULL,NULL};
			gint status;
			gint ret;
			gchar *section;

			argv[1] = (gchar *)value;

			section = strchr(value,'/');
			if (section) {
				*section = '\0';
				section++;
				argv[2] = (gchar *)section;
				argv[3] = tmp_file;
			} else {
				argv[2] = tmp_file;
			}

			ret = g_spawn_sync(NULL,argv,NULL,G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_STDOUT_TO_DEV_NULL,NULL,NULL,NULL,NULL,&status,NULL);

			if (ret == TRUE && status == 0) {
				gtk_moz_embed_load_url(moz,tmp_file);
			}

			return TRUE;
		}
	}

	return FALSE;
}

static void rman_close(IshzillaModule *module)
{
	unlink(tmp_file);

	g_free(tmp_file);

	return;
}


